/**
 * Copyright (c) 2000-2011 Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.wsrp.util;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.InstanceFactory;
import com.liferay.portal.model.User;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import oasis.names.tc.wsrp.v2.types.UserProfile;

import org.apache.axis.message.MessageElement;

/**
 * @author Michael C. Han
 */
public class ConsumerRequestExtensionsHelper {

	public static void addClientAttributes(
		List<MessageElement> clientAttributes) {

		_instance._addClientAttributes(clientAttributes);
	}

	public static void addUserProfileAttributes(
		UserProfile userProfile, User user) {

		_instance._addUserProfileAttributes(userProfile, user);
	}

	private ConsumerRequestExtensionsHelper() {
		try {
			_initConsumerRequestExtensions();
		}
		catch (Exception e) {
			_log.error("Unable to instantiate consumer request extension", e);

			throw new RuntimeException(e);
		}
	}

	private void _addClientAttributes(List<MessageElement> clientAttributes) {
		for (ConsumerRequestExtension consumerRequestExtension :
				_consumerRequestExtensions) {

			consumerRequestExtension.addClientAttributes(clientAttributes);
		}
	}

	private void _addUserProfileAttributes(UserProfile userProfile, User user) {
		for (ConsumerRequestExtension consumerRequestExtension :
				_consumerRequestExtensions) {

			consumerRequestExtension.addUserProfileAttributes(
				userProfile, user);
		}
	}

	private void _initConsumerRequestExtensions() throws Exception {
		if (PortletPropsValues.CONSUMER_REQUEST_EXTENSIONS.length == 0) {
			return;
		}

		_consumerRequestExtensions = new ArrayList<ConsumerRequestExtension>(
			PortletPropsValues.CONSUMER_REQUEST_EXTENSIONS.length);

		for (String consumerRequestExtensionClassName :
				PortletPropsValues.CONSUMER_REQUEST_EXTENSIONS) {

			ConsumerRequestExtension consumerRequestExtension =
				(ConsumerRequestExtension)InstanceFactory.newInstance(
					consumerRequestExtensionClassName);

			_consumerRequestExtensions.add(consumerRequestExtension);
		}
	}

	private static Log _log = LogFactoryUtil.getLog(
		ConsumerRequestExtensionsHelper.class);

	private static ConsumerRequestExtensionsHelper _instance =
		new ConsumerRequestExtensionsHelper();

	private List<ConsumerRequestExtension> _consumerRequestExtensions =
		Collections.emptyList();

}